#!/usr/bin/env bash
# $Id: check_cluster.sh,v 1.10 2011/02/26 19:23:08 bzfpfets Exp $
#
# Call with "make testcluster"
#
# To get the result files call "./evalcheck_cluster.sh
# results/check.$TSTNAME.$BINNMAE.$SETNAME.eval in directory check/
# This leads to result files 
#  - results/check.$TSTNAME.$BINNMAE.$SETNAME.out
#  - results/check.$TSTNAME.$BINNMAE.$SETNAME.res
#  - results/check.$TSTNAME.$BINNMAE.$SETNAME.err
#
TSTNAME=$1
BINNAME=$2
SETNAME=$3
BINID=$4
TIMELIMIT=$5
NODELIMIT=$6
MEMLIMIT=$7
DISPFREQ=$8
VERSION=${9}
LPS=${10}
QUEUE=${11}
PPN=${12}
CLIENTTMPDIR=${13}
SEED=${14}

ACCOUNT="dopt"
EXCLUSIVE="--exclusive"
QUEUETYPE="sbatch"

# get current path
CURPATH=`pwd`

if test ! -e $CURPATH/results
then
    mkdir $CURPATH/results
fi

SETDIR=$CURPATH/../settings

# set settings name if necessary
if test "$SETNAME" = "default"
then
    SETTINGS=""
else
    SETTINGS=$SETDIR"/"$SETNAME".set"
fi

# check if the settings file exists
if test $SETNAME != "default"
then
    if test ! -e $SETTINGS
    then
	echo skipping test due to the non-existence of the settings file $SETTINGS
	exit
    fi
fi

# check if queue has been defined
if test "$QUEUE" = ""
then
    echo Skipping test since the queue name has not been defined.
    exit
fi

# check if number of nodes has been defined
if test "$PPN" = ""
then
    echo Skipping test since the number of nodes has not been defined.
    exit
fi

# check if client tmp-dir has been defined
if test "$CLIENTTMPDIR" = ""
then
    echo Skipping test since the path for the tmp-dir on the clients has not been defined.
    exit
fi

# we add 200% to the hard time limit and additional 600 seconds in case of small time limits
HARDTIMELIMIT=`expr $TIMELIMIT + $TIMELIMIT`
HARDTIMELIMIT=`expr $HARDTIMELIMIT + $TIMELIMIT`
HARDTIMELIMIT=`expr $HARDTIMELIMIT + 600` 

# we add 10% to the hard memory limit and additional 100mb to the hard memory limit
# the srun queue requires a format duration HH:MM:SS (and optionally days) and megabytes,
# whereas the qsub requires the memory limit in kB
if test "$QUEUETYPE" != "qsub"
then
    # values of MEMLIMIT are in MB
    HARDMEMLIMIT=`expr \`expr $MEMLIMIT + 100\` + \`expr $MEMLIMIT / 10\``

    #format is (d-)HH:MM:SS
    TMP=`expr $HARDTIMELIMIT`
    HARDTIMELIMIT=""
    DIVISORS=(60 60 24)
    for((i=0; i<=2; i++))
    do
        printf -v HARDTIMELIMIT "%02d${HARDTIMELIMIT}" `expr ${TMP} % ${DIVISORS[i]}`
        # separate the numbers by colons except for the last (HH hours)
        if test $i -lt 2
        then
            HARDTIMELIMIT=":${HARDTIMELIMIT}"
        fi
        TMP=`expr ${TMP} / ${DIVISORS[i]}`
    done
    if test ${TMP} -gt 0
    then
        HARDTIMELIMIT=${TMP}-${HARDTIMELIMIT}
    fi
else
    # values of MEMLIMIT are in MB
    # values of HARDMEMLIMIT are in byte for qsub
    HARDMEMLIMIT=`expr \`expr $MEMLIMIT + 100\` + \`expr $MEMLIMIT / 10\``
    HARDMEMLIMIT=`expr $HARDMEMLIMIT \* 1024000`
fi

if test $SEED == -1
then
    EVALFILE=$CURPATH/results/check.$QUEUE.$TSTNAME.$BINID.$SETNAME.eval
else
    EVALFILE=$CURPATH/results/check.$QUEUE.$TSTNAME.$BINID.$SETNAME."s"$SEED.eval
fi
echo > $EVALFILE

echo "testset        " $TSTNAME
echo "queue          " $QUEUE
echo "memlimit       " $MEMLIMIT
echo "hard memlimit  " $HARDMEMLIMIT
echo "timelimit      " $TIMELIMIT
echo "hard timelimit " $HARDTIMELIMIT
echo "seed           " $SEED
echo

# counter to define file names for a test set uniquely
COUNT=1

STOREFILENAME=""
for i in `cat testset/$TSTNAME.test` DONE
do
    if test "$i" = "DONE"
    then
	break
    fi

    if test "$STOREFILENAME" == ""
    then
	STOREFILENAME=$i
	continue
    fi
    INSTANCE=$STOREFILENAME
    STOREFILENAME=""

    if test "$i" == "-"
    then
	TESTINST=""
    else
	TESTINST=$i
    fi

    SHORTFILENAME=`basename $INSTANCE .gz`
    
    if test $SEED == -1
    then
	BASENAME=$USER.$QUEUE.$TSTNAME.$COUNT"_"$SHORTFILENAME.$BINID.$SETNAME
    else
	BASENAME=$USER.$QUEUE.$TSTNAME.$COUNT"_"$SHORTFILENAME.$BINID.$SETNAME."s"$SEED
    fi

    SETFILE=$CURPATH/results/$BASENAME.set

    echo $CURPATH/results/$BASENAME >> $EVALFILE

    JOBNAME="IPB-${SHORTFILENAME}"

    export CLIENTTMPDIR
    export INSTANCE
    export TESTINST
    export SETTINGS
    export CURPATH
    export BINNAME
    export TIMELIMIT
    export MEMLIMIT
    export NODELIMIT
    export DISPFREQ
    export BASENAME
    export SEED

    # echo sbatch --job-name=${JOBNAME} --mem=$HARDMEMLIMIT -p $QUEUE -A $ACCOUNT --time=${HARDTIMELIMIT} ${EXCLUSIVE} --output=/dev/null runclcluster.sh
    sbatch --job-name=${JOBNAME} --mem=$HARDMEMLIMIT -p $QUEUE -A $ACCOUNT --time=${HARDTIMELIMIT} ${EXCLUSIVE} --output=/dev/null runclcluster.sh

    COUNT=`expr $COUNT + 1`
done
