#!/usr/bin/env bash
# $Id: check_cluster.sh,v 1.10 2011/02/26 19:23:08 bzfpfets Exp $
#
# Call with "make testcluster"
#
# To get the result files call "./evalcheck_cluster.sh
# results/check.$TSTNAME.$BINNMAE.$SETNAME.eval in directory check/
# This leads to result files 
#  - results/check.$TSTNAME.$BINNMAE.$SETNAME.out
#  - results/check.$TSTNAME.$BINNMAE.$SETNAME.res
#  - results/check.$TSTNAME.$BINNMAE.$SETNAME.err
#
TSTNAME=$1
BINNAME=$2
SETNAME=$3
BINID=$4
TIMELIMIT=$5
NODELIMIT=$6
MEMLIMIT=$7
DISPFREQ=$8
VERSION=${9}
LPS=${10}
QUEUE=${11}
PPN=${12}
CLIENTTMPDIR=${13}

# get current path
CURPATH=`pwd`

if test ! -e $CURPATH/results
then
    mkdir $CURPATH/results
fi

SETDIR=$CURPATH/../settings

# set settings name if necessary
if test "$SETNAME" = "default"
then
    SETTINGS=""
else
    SETTINGS=$SETDIR"/"$SETNAME".set"
fi

# check if the settings file exists
if test $SETNAME != "default"
then
    if test ! -e $SETTINGS
    then
	echo skipping test due to the non-existence of the settings file $SETTINGS
	exit
    fi
fi

# check if queue has been defined
if test "$QUEUE" = ""
then
    echo Skipping test since the queue name has not been defined.
    exit
fi

# check if number of nodes has been defined
if test "$PPN" = ""
then
    echo Skipping test since the number of nodes has not been defined.
    exit
fi

# check if client tmp-dir has been defined
if test "$CLIENTTMPDIR" = ""
then
    echo Skipping test since the path for the tmp-dir on the clients has not been defined.
    exit
fi

# we add 100% to the hard time limit and additional 600 seconds in case of small time limits
# NOTE: the jobs should have a hard running time of more than 5 minutes; if not so, these
#       jobs get automatically assigned in the "exrpess" queue; this queue has only 4 CPUs
#       available
HARDTIMELIMIT=`expr \`expr $TIMELIMIT + 600\` + $TIMELIMIT`

# we add 10% to the hard memory limit and additional 100mb to the hard memory limit
HARDMEMLIMIT=`expr \`expr $MEMLIMIT + 100\` + \`expr $MEMLIMIT / 10\``
HARDMEMLIMIT=`expr $HARDMEMLIMIT \* 1024000`

EVALFILE=$CURPATH/results/check.$QUEUE.$TSTNAME.$BINID.$SETNAME.eval
echo > $EVALFILE

echo "testset        " $TSTNAME
echo "queue          " $QUEUE
echo "memlimit       " $MEMLIMIT
echo "hard memlimit  " $HARDMEMLIMIT
echo "timelimit      " $TIMELIMIT
echo "hard timelimit " $HARDTIMELIMIT
echo

# counter to define file names for a test set uniquely
COUNT=1

SOLUFILE=testset/$TSTNAME.solu

for i in `cat testset/$TSTNAME.test` DONE
do
  if test "$i" = "DONE"
  then
      break
  fi

  INSTANCE=$CURPATH/$i

  # check if problem instance exists
  if test -e $INSTANCE
  then
      SHORTFILENAME=`basename $i .gz`
      SHORTFILENAME=`basename $SHORTFILENAME .mps`
      SHORTFILENAME=`basename $SHORTFILENAME .lp`
      SHORTFILENAME=`basename $SHORTFILENAME .opb`
      SHORTFILENAME=`basename $SHORTFILENAME .col`

      BASENAME=$USER.$QUEUE.$TSTNAME.$COUNT"_"$SHORTFILENAME.$BINID.$SETNAME

      SETFILE=$CURPATH/results/$BASENAME.set

      # get objective bound
      if test -e "$SOLUFILE"
      then
	  OBJECTIVEVAL=`grep "$SHORTFILENAME " $SOLUFILE | grep -v =feas= | grep -v =inf= | tail -n 1 | awk '{print $3}'`
      else
	  OBJECTIVEVAL=""
      fi

      if test "$OBJECTIVEVAL" == ""
      then
	  echo "Could not find objective value for problem "$SHORTFILENAME
      else
	  echo $CURPATH/results/$BASENAME >> $EVALFILE

	  qsub -l walltime=$HARDTIMELIMIT -l mem=$HARDMEMLIMIT -l nodes=1:ppn=$PPN -N $SHORTFILENAME -v CURPATH=$CURPATH,BINNAME=$BINNAME,INSTANCE=$INSTANCE,BASENAME=$BASENAME,SETTINGS=$SETTINGS,TIMELIMIT=$TIMELIMIT,MEMLIMIT=$MEMLIMIT,NODELIMIT=$NODELIMIT,DISPFREQ=$DISPFREQ,CLIENTTMPDIR=$CLIENTTMPDIR,OBJLIMIT=$OBJECTIVEVAL -q $QUEUE -o /dev/null -e /dev/null runclcluster.sh

	  COUNT=`expr $COUNT + 1`
      fi
  else
      echo "input file "$INSTANCE" not found!"
  fi
done
