#!/usr/bin/env bash

# create table for all relevant variants
#
DATADIR="results-paper-version2"

if test ! -e $DATADIR
then
    echo "Please create link to results data."
    exit
fi

# output file
FILE=table-libsvm-overview.tex
echo "Writing file "$FILE
echo "% generated by paper_generate_overview_table.sh" > $FILE
echo "\begin{table*}[t]" >> $FILE
echo "   \small" >> $FILE
echo "   \caption{Aggregated results for LIBSVM: Average test/train accuracies and standard deviations (STD) for three algorithms over " >> $FILE
echo "            10 different seeds, using class probabilities for prediction and voting; we considered 40 instances " >> $FILE
echo "            as outlined in Section~\ref{sec:comp-results}. Column \`\`\#~best'' represents the number of instances " >> $FILE
echo "            on which the corresponding algorithm performed best (ties possible). Column \`\`ER'' gives the error rate," >> $FILE
echo "            i.e., \(1/(1-a)\) for the average accuracy~\(a\).}" >> $FILE
echo "   \label{tab:aggregated}" >> $FILE
echo "   \begin{tabular*}{\textwidth}{@{}l@{\quad\extracolsep{\fill}}rrrrrrrrrrrrr@{}}\toprule" >> $FILE
echo "      && \multicolumn{4}{c}{IPBoost} & \multicolumn{4}{c}{LPBoost} & \multicolumn{4}{c}{AdaBoost}\\\\" >> $FILE
echo "         \cmidrule(lr){3-6}\cmidrule(lr){7-10}\cmidrule(l){11-14}" >> $FILE
echo "      type & \(\rho\)   & \# best & accuracy & STD & ER & \# best & accuracy & STD & ER & \# best & accuracy & STD & ER \\\\" >> $FILE
echo "      \midrule" >> $FILE

# settings
ACC=(0.1 0.075 0.05 0.025 0.01)

# first testing
for A in ${ACC[@]}
do
    AA=`echo "$A" | sed "s/\./_/"`

    perl generate_overview_table.prl $DATADIR/check.prio.libsvm.ipboost.linux.x86_64.gnu.opt.spx2.moskito.margin"$AA".s*.out >> $FILE
done

echo "      \midrule" >> $FILE

# now training
for A in ${ACC[@]}
do
    AA=`echo "$A" | sed "s/\./_/"`

    perl generate_overview_table.prl -t $DATADIR/check.prio.libsvm.ipboost.linux.x86_64.gnu.opt.spx2.moskito.margin"$AA".s*.out >> $FILE
done

echo "      \bottomrule" >> $FILE
echo "   \end{tabular*}" >> $FILE
echo "\end{table*}" >> $FILE
