#!/usr/bin/env bash

# create table for all relevant variants
#
DATADIR="results-paper-version2"

if test ! -e $DATADIR
then
    echo "Please create link to results data."
    exit
fi

# runtimes only for rho = 0.05
FILE=table-libsvm-runtimes.tex
echo "Writing file "$FILE

echo "% generated by paper_generate_pred_tables.sh" > $FILE
echo "\begin{table}" >> $FILE
echo "   \small" >> $FILE
echo "   \caption{Statistics for LIBSVM on average run times (in seconds) of different algorithms with $\rho = 0.05$ for 10 different seeds;" >> $FILE
echo "            \`\`\\#~optimal'' gives the number of instances solved to optimality, \`\`\\#~time out'' the number of instances that ran into the" >> $FILE
echo "            time limit of 1 hour, and \`\`best sol.\ time'' the average time at which the best solution was found.}" >> $FILE
echo "   \label{tab:runtimeStats}" >> $FILE
echo "   \begin{tabular*}{\textwidth}{@{}l@{\extracolsep{\fill}}rrrrrrr@{}}\toprule" >> $FILE
echo "      & \multicolumn{4}{c}{IPBoost} & &  \\\\ \cmidrule(r){2-6}" >> $FILE
echo "      name & total time & \# nodes & \# optimal & \# time out & best sol.\ time & LPBoost time & AdaBoost time \\\\ \midrule" >> $FILE

perl generate_runtimes_mean.prl $DATADIR/check.prio.libsvm.ipboost.linux.x86_64.gnu.opt.spx2.moskito.margin0_05.s*.out >> $FILE

echo "      \bottomrule" >> $FILE
echo "   \end{tabular*}" >> $FILE
echo "\end{table}" >> $FILE


# settings
ACC=(0.1 0.075 0.05 0.025 0.01)

# generate tables
for A in ${ACC[@]}
do
    AA=`echo "$A" | sed "s/\./_/"`

    # testing
    FILE=table-libsvm-test-$AA.tex
    echo "Writing file "$FILE

    echo "% gnerated by paper_generate_pred_tables.sh" > $FILE
    echo "\begin{table}" >> $FILE
    echo "   \small" >> $FILE
    echo "   \caption{Averages of the \emph{test} accuracies and standard deviations for three algorithms" >> $FILE
    echo "            with $\rho = "$A"$ for 10 different seeds on LIBSVM; best solutions are marked with ``*``; using" >> $FILE
    echo "            class probabilities for prediction and voting.}" >> $FILE
    echo "   \label{tab:libsvm:test:"$AA"}" >> $FILE
    echo "   \begin{tabular*}{\textwidth}{@{}l@{\extracolsep{\fill}}r@{\extracolsep{0ex}\quad}r@{\;}>{$}c<{$}@{\;}r@{\quad}r@{\extracolsep{\fill}}r@{\extracolsep{0ex}\quad}r@{\;}>{$}c<{$}@{\;}r@{\quad}r@{\extracolsep{\fill}}r@{\extracolsep{0ex}\quad}r@{\;}>{$}c<{$}@{}r@{\quad}r@{}}\toprule">> $FILE
    
    perl generate_table_mean.prl $DATADIR/check.prio.libsvm.ipboost.linux.x86_64.gnu.opt.spx2.moskito.margin"$AA".s*.out >> $FILE

    echo "      \bottomrule" >> $FILE
    echo "   \end{tabular*}" >> $FILE
    echo "\end{table}" >> $FILE

    # training
    FILE=table-libsvm-train-$AA.tex
    echo "Writing file "$FILE

    echo "% gnerated by paper_generate_pred_tables.sh" > $FILE
    echo "\begin{table}" >> $FILE
    echo "   \small" >> $FILE
    echo "   \caption{Averages of the \emph{train} accuracies and standard deviations for three algorithms" >> $FILE
    echo "            with $\rho = "$A"$ for 10 different seeds on LIBSVM; best solutions are marked with ``*``; using" >> $FILE
    echo "            class probabilities for prediction and voting.}" >> $FILE
    echo "   \label{tab:libsvm:train:"$AA"}" >> $FILE
    echo "   \begin{tabular*}{\textwidth}{@{}l@{\extracolsep{\fill}}r@{\extracolsep{0ex}\quad}r@{\;}>{$}c<{$}@{\;}r@{\quad}r@{\extracolsep{\fill}}r@{\extracolsep{0ex}\quad}r@{\;}>{$}c<{$}@{\;}r@{\quad}r@{\extracolsep{\fill}}r@{\extracolsep{0ex}\quad}r@{\;}>{$}c<{$}@{}r@{\quad}r@{}}\toprule">> $FILE
    
    perl generate_table_mean.prl -t $DATADIR/check.prio.libsvm.ipboost.linux.x86_64.gnu.opt.spx2.moskito.margin"$AA".s*.out >> $FILE

    echo "      \bottomrule" >> $FILE
    echo "   \end{tabular*}" >> $FILE
    echo "\end{table}" >> $FILE
done
