# Flip noise to given problems by switching a certain fraction of the labels.
#
import sys
from sklearn.datasets import load_svmlight_file
from sklearn.datasets import dump_svmlight_file
from sklearn import preprocessing
import numpy as np
import os

if len(sys.argv) != 3:
    print("usage: <.> <libsvm file> <noise level>")
    exit()

print("Reading: " + sys.argv[1])
features, labels = load_svmlight_file(sys.argv[1], dtype='double')

# adjust labels to be in {0,1} (for two classes)
le = preprocessing.LabelEncoder()
labels = le.fit_transform(labels)
assert len(le.classes_) <= 2


# apply noise to labels
noise = float(sys.argv[2])
print("Noise: " + str(noise))

npoints = len(labels)
print("Number of points: " + str(npoints))

# generate subset of indices
s = int(noise * npoints);
print("Number of affected points: " + str(s))
index = np.random.choice(np.arange(0, npoints-1), size=s, replace=False)

for i in range(len(index)):
    idx = index[i]
    labels[idx] = 1.0 - labels[idx]

probname = os.path.basename(sys.argv[1])
probname = os.path.splitext(probname)[0]
probname = probname + "_e0_1"
print("Writing file to " + probname)
dump_svmlight_file(features, labels, probname)
