import supervised
import utils
import globs
import os


import matplotlib.pyplot as plt
from sklearn import datasets
from sklearn.utils import shuffle

def printSamplePred(digits, data, classifier, cname):
    n_samples = len(digits.images)

    expected = digits.target[:]
    predicted = classifier.predict(data[:])

    images_and_predictions = list(zip(digits.images[:], predicted))
    for index, (image, prediction) in enumerate(images_and_predictions[:4]):
        plt.subplot(2, 4, index + 5)
        plt.axis('off')
        plt.imshow(image, cmap=plt.cm.gray_r, interpolation='nearest')
        plt.title('Prediction: %i' % prediction)

    plt.savefig(os.getcwd() + '/graphs/' + cname + '.pdf')
    plt.close()



# The digits dataset
digits = datasets.load_digits()

# preprocessing the data for the test
digits.images, digits.target = shuffle(digits.images, digits.target, random_state = globs.rstate)

images_and_labels = list(zip(digits.images, digits.target))
for index, (image, label) in enumerate(images_and_labels[:4]):
    plt.subplot(2, 4, index + 1)
    plt.axis('off')
    plt.imshow(image, cmap=plt.cm.gray_r, interpolation='nearest')
    plt.title('Training: %i' % label)

# To apply a classifier on this data, we need to flatten the image, to
# turn the data in a (samples, feature) matrix:
# this is happening here

n_samples = len(digits.images)
data = digits.images.reshape((n_samples, -1))

globs.compAuC = 0
globs.plotROC = 0

#######################################################################
# Training a random forest and a support vector machine classifier
#######################################################################

cRF = supervised.trainClassifier(digits.target[:n_samples], data[:n_samples], globs.tRF)
cSVM = supervised.trainClassifier(digits.target[:n_samples], data[:n_samples], globs.tSVM)
cGNB = supervised.trainClassifier(digits.target[:n_samples], data[:n_samples], globs.tGaussNB)
cADA = supervised.trainClassifier(digits.target[:n_samples], data[:n_samples], globs.tAdaBoost)

# we predict on _all_ trained and validation
# this is for _test purposes_ only!!

printSamplePred(digits, data, cRF, 'RF')
printSamplePred(digits, data, cSVM, 'SVM')
printSamplePred(digits, data, cGNB, 'GNB')
printSamplePred(digits, data, cADA, 'DecT')
