import supervised
from sklearn import metrics
import sklearn
import os
import numpy
import pickle
import gzip
import random
import matplotlib.pyplot as plt
from sklearn import datasets
from sklearn.utils import shuffle
# from sklearn.ensemble import VotingClassifier
from votingClass import VotingClassifier
import math
import copy
import sklearn.datasets
from sklearn.model_selection import cross_val_score
from sklearn import model_selection
from sklearn.ensemble import RandomForestClassifier, RandomForestRegressor, AdaBoostClassifier, GradientBoostingClassifier, ExtraTreesClassifier

from sklearn.datasets.mldata import fetch_mldata
import tempfile



import utils
import globs
import tob



def printFunc(str):
    if globs.debugoutput == 1:
        print(str)

def resampleData(dist):
    numpy.random.seed(globs.rstate)
    onorm = sum(dist)
    dist = [x / onorm for x in dist]
    index = numpy.random.choice(numpy.arange(0, len(dist)), size=globs.subSam, p=dist, replace=True)
    return index

def normalizeDist(dist):
    onorm = sum(dist)
    dist = [x / onorm for x in dist]
    return dist



def unitTestRaw():
    digits = datasets.load_digits()
    dist = [1 for x in range(len(digits.images))]
    index = resampleData(dist)

    # full data for benchmark
    fFeatures = digits.images
    fLabels = digits.target
    n_samples = len(fFeatures)
    fData = fFeatures.reshape((n_samples, -1))

    # subsample data for training with training distribution
    features = digits.images[index]
    labels = digits.target[index]
    n_samples = len(features)
    data = features.reshape((n_samples, -1))


    # print(features.shape)
    # print(data.shape)

    # preprocessing the data for the test
    # images = digits.images[index]
    # digits.images, digits.target = shuffle(digits.images, digits.target, random_state = globs.rstate)


    globs.compAuC = 0
    globs.plotROC = 0
    globs.genClassReport = 0

    cRF = supervised.trainClassifier(labels, data, globs.tRF)
    pred = cRF.predict(fData)

    print("Confusion matrix:\n\n%s" % metrics.confusion_matrix(pred, fLabels))


def sampleAndTrain(dist, features, labels, rstate):
    globs.rstate = rstate

    index = resampleData(dist)

    # sub-sampled data according to distribution
    subFeatures = features[index]
    subLabels = labels[index]

    globs.compAuC = 0
    globs.plotROC = 0
    globs.genClassReport = 0

    cRF = supervised.trainClassifier(subLabels, subFeatures, globs.tLearn)
    # cRF = supervised.trainClassifier(subLabels, subFeatures, globs.tDecTree)


    # we can work directly with the probabilities
    # have to filter the probabilities for the 1-clas
    # check classes_ for the ordering of the class labels. index 1 should correspond to class 1
    # print(cRF.classes_)

    # pred = cRF.predict_proba(features)
    # pred = [x[1] for x in pred]

    # direct predictions not probabilites

    return cRF


def generateCol(dist, features, labels, rstate):
    # fixing the seed for determinism as well as for later to handle data.
    cRF = sampleAndTrain(dist, features, labels, rstate)

    pred = cRF.predict(features)

    pred = [ (x*2 - 1) * (y*2 - 1) for (x,y) in zip(pred,labels)]

    return pred

def buildVoter(weights, distribs, features, labels, rstate):
    trainedLearners = []
    estimators = []

    print("Fitting base classifier for benchmark...")
    cRF = supervised.trainClassifier(labels, features, globs.tRF)

    print("\nBuilding and training voting classifier...")

    for i, dist in enumerate(distribs):
        cRF = sampleAndTrain(dist,features,labels,rstate)
        trainedLearners.append(cRF)
        estimators.append(["est" + str(i), cRF])
    # TODO: we can experiment with the voting later
    clf = VotingClassifier(estimators=estimators, voting='hard', weights=weights)
    return clf

def simpleBoost(features, labels, T=10, rstate=0):
    # start with a uniform prior
    globs.tLearn = globs.tDecTree
    eps = 0.01
    dist = [1 for i in range(len(labels))]
    # determine learning rate
    eta = math.sqrt(2/T * math.log(1/eps))
    estimators = []
    for i in range(T):
        print("\n---> Boosting iteration: ", str(i))
        cRF = sampleAndTrain(dist, features, labels, rstate)
        pred = cRF.predict(features)
        pred = [1/2 - ((x * 2 - 1) * (y * 2 - 1))/2 for (x, y) in zip(pred, labels)]
        dist = [p * math.exp(eta*r) for (p,r) in zip(dist,pred)]
        dist = normalizeDist(dist)
        # print(dist)
        saveL = cRF # Deep copy seems to be unnecessary copy.deepcopy(cRF)
        estimators.append(["est" + str(i), saveL])
        # print(dist)
        clf = VotingClassifier(estimators=estimators, voting='hard')
        score = clf.score(features, labels)
        print("Current boosted accuracy: " + '{:.2f}'.format(score))
    # clf = VotingClassifier(estimators=estimators, voting='hard')
    # score = clf.score(features, labels)
    print("")
    print("--> Final boosted accuracy: " + '{:.2f}'.format(score))



def unitTest():
    # read data from test file
    # features, labels = generateData("test")
    # features, labels = generateData("")

    #features, labels = generateData("breast_cancer")
    features, labels = generateData("Digit1")
    labels = numpy.squeeze(numpy.asarray(labels))
    print("Feature dimension:", features.shape)
    print("Label dimension:", labels.shape)

    # arbitrary test distribution to see whether we can sample
    dist = [random.random() for x in range(len(features))]

    # pred = generateCol(dist,features,labels, globs.rstate)
    # # print(pred)
    #
    # distribs = []
    # K = 10
    # for i in range(K):
    #     dist = [random.random() for x in range(len(features))]
    #     distribs.append(dist)
    # distribs = numpy.array(distribs)
    # weights = [random.random() for i in range(K)]
    # clf = buildVoter(weights, distribs, features, labels, globs.rstate)
    # score = clf.score(features,labels)
    # print("Score: " + '{:2f}'.format(score))
    #     # this is from a test: keep for now. the output has now a shifted form to incorporate the identity test.
    # # print("Confusion matrix:\n\n%s" % metrics.confusion_matrix(pred, labels))

    # simpleBoost(features,labels,T=10)
    clf = tob.treeOfboostersClassifier()
    clf = clf.fit(features,labels)


#######################################################
#### DATA MANAGEMENT FUNCTIONS
#######################################################


def generateHardDistribution(gamma,examples,noise):
    baseD = [[1.0,0.0,1.0], [gamma,-gamma,1.0], [gamma,-gamma,1.0], [gamma,5.0*gamma,1.0]]
    numpy.random.seed(globs.rstate)
    index = numpy.random.choice(numpy.arange(0, len(baseD)), size=examples, replace=True)
    npBaseD = numpy.matrix(baseD)
    upsample = npBaseD[index]
    # print(index)
    # print(upsample)
    features = upsample[:,[0,1]]
    labels = upsample[:,2]
    labels = numpy.squeeze(numpy.asarray(labels))
    # labels = numpy.squeeze(labels) *+*
    for i in range(labels.shape[0]):
        if random.random() < noise:
            labels[i] = (-1.0 * labels[i] + 1.0)/2.0
        else:
            labels[i] = (labels[i] + 1.0)/2.0
    # print(labels)
    # print(features)
    #print(labels.shape)
    return features,labels

def generateData(filename):
    if filename == "":
#        features, labels = sklearn.datasets.make_classification(n_samples=100)
        features, labels = sklearn.datasets.make_classification(n_samples=1000, flip_y=0.01, n_features=50, n_informative=5)
    elif filename == "breast_cancer":
        features, labels = datasets.load_breast_cancer(True)
    elif filename == "UCI-VOTE":
        # TODO: fix data set reading from mldata.org we need
        # BCW (already there: breast_cancer), VOTE,CLVHEART, HUHEART, SONAR)
        test_data_home = tempfile.mkdtemp()
        raw = fetch_mldata('datasets-UCI vote', data_home=test_data_home)
        print(raw)
    elif filename == "Digit1":
        digits = datasets.load_digits()
        digits.images, digits.target = shuffle(digits.images, digits.target, random_state=globs.rstate)
        n_samples = len(digits.images)
        data = digits.images.reshape((n_samples, -1))
        labels = digits.target[:n_samples]
        features = data[:n_samples]
        labels = [0 if lab < 5 else 1 for lab in labels]
    else:
        features, labels = loadData(filename)
    npFeatures = numpy.matrix(features)
    npLabels = numpy.matrix(labels)
    return npFeatures, npLabels


def loadData(filename):
    printFunc("--> Loading data from " + filename + ".dtz...")
    f = gzip.open(filename + '.dtz', 'rb')
    features, labels = pickle.load(f)
    f.close()
    printFunc("Data points loaded: " + str(len(labels)))
    return features, labels

def unitTestData():
    # create a dummy data set
    utils.generateDataStore()
    # check whether we can read-in our dummy data set
    # features,labels = generateData("test")

    features,labels = generateData("UCI-VOTE")

    # print(features)
    # print(labels)


def unitTestHardD():
    features, labels = generateHardDistribution(0.1,100,0.2)
    # print(features)
    # print(labels)
    simpleBoost(features, labels, T=100)
    # clf = tob.treeOfboostersClassifier()
    # clf = clf.fit(features,labels)

def testNewClassifier():
    features, labels = generateHardDistribution(0.1,100,0.2)

    # features, labels = generateData("breast_cancer")
    # labels = numpy.squeeze(numpy.asarray(labels))


    print("Feature dimension:", features.shape)
    print("Label dimension:", labels.shape)

    # simpleBoost(features, labels, T=10)
    # print(labels)

    #clf = tob.treeOfboostersClassifier(type=tob.tobForest)
    clf = tob.treeOfboostersClassifier()
    # clf = clf.fit(features,labels)
    # test = clf.predict(features)


    Xtrain, Xtest, ytrain, ytest = model_selection.train_test_split(features, labels, test_size=0.5,
                                                                    random_state=globs.rstate)

    clf = clf.fit(Xtrain, ytrain,T=10)

    print("\n========= TOB PERFORMANCE ===================")
    val = clf.score(Xtrain, ytrain)
    printFunc("Train (CV) accuracy: %0.3f" % val)
    val = clf.score(Xtest, ytest)
    printFunc("Held out cross validation (CV) accuracy: %0.3f" % val)

    # TODO: for this to work implement get_params (as probably a few other functions in the tob class)
    # score = cross_val_score(clf, features, labels, cv=globs.numCvs)
    # val = [score.mean(), score.std() * 2]
    # printFunc("Accuracy: %0.3f (+/- %0.3f)" % (val[0], val[1]))

    clf = RandomForestClassifier()
    clf = clf.fit(Xtrain, ytrain)

    print("\n=========== RF PERFORMANCE ===================")
    val = clf.score(Xtrain, ytrain)
    printFunc("Train (CV) accuracy: %0.3f" % val)
    val = clf.score(Xtest, ytest)
    printFunc("Held out cross validation (CV) accuracy: %0.3f" % val)

    # for this to work implement get_params
    # score = cross_val_score(clf, features, labels, cv=globs.numCvs)
    # val = [score.mean(), score.std() * 2]
    # printFunc("Accuracy: %0.3f (+/- %0.3f)" % (val[0], val[1]))


#######################################################
#### UNIT TESTS BEFORE INTEGRATIING
#######################################################

# unitTest()
# unitTestData()

unitTestHardD()

# testNewClassifier()