import numpy as np
from sklearn.utils.validation import check_is_fitted
from sklearn.metrics import accuracy_score

import globs
import math
import supervised
from sklearn.ensemble import RandomForestClassifier
from sklearn.tree import DecisionTreeClassifier

tobForest = 1
tobTree = 0

def printFunc(str):
    if globs.debugoutput == 1:
        print(str)

def resampleData(dist):
    np.random.seed(globs.rstate)
    onorm = sum(dist)
    dist = [x / onorm for x in dist]
    index = np.random.choice(np.arange(0, len(dist)), size=globs.subSam, p=dist, replace=True)
    return index

def normalizeDist(dist):
    onorm = sum(dist)
    dist = [x / onorm for x in dist]
    return dist

def sampleAndTrain(dist, features, labels, rstate):
    globs.rstate = rstate

    index = resampleData(dist)

    # sub-sampled data according to distribution
    subFeatures = features[index]
    subLabels = labels[index]

    globs.compAuC = 0
    globs.plotROC = 0
    globs.genClassReport = 0

    cRF = supervised.trainClassifier(subLabels, subFeatures, globs.tLearn)
    return cRF


class treeOfboostersClassifier(object):
    """Forest/Tree of Boosters classification"""

    def __init__(self, type=tobTree,eps=0.01):
        self.type = type
        if type == tobForest:
            self._tree = RandomForestClassifier()
        elif type == tobTree:
            self._tree = DecisionTreeClassifier()
        self._estimators = []
        self._boostFet = []
        self._eps = eps
        self._boostAcc = None


    def fit(self, X, y, T=10):
        # start with a uniform prior
        globs.tLearn = globs.tDecTree
        eps = self._eps
        dist = [1 for i in range(len(y))]
        # determine learning rate
        eta = math.sqrt(2 / T * math.log(1 / eps))
        for i in range(T):
            print("\n---> Boosting iteration: ", str(i))
            # TODO: later replace by direct training
            cRF = sampleAndTrain(dist, X, y, globs.rstate)
            predVanilla = cRF.predict(X)
            # print("before flipping predicted", predVanilla)
            pred = [1.0 / 2.0 - ((x * 2.0 - 1.0) * (y * 2.0 - 1.0)) / 2.0 for (x, y) in zip(predVanilla, y)]
            # print("after flipping predicted (0 is correct / 1 is not)", pred)
            dist = [p * math.exp(eta * r) for (p, r) in zip(dist, pred)]
            dist = normalizeDist(dist)
            self._estimators.append(cRF)
            self._boostFet.append(predVanilla)
            boostFeatures = np.asarray(self._boostFet)
            boostFeatures = np.transpose(boostFeatures)
            self._tree = self._tree.fit(boostFeatures,y)
            predTree = self._tree.predict(boostFeatures)
            predTree = [1 / 2 - ((x * 2 - 1) * (y * 2 - 1)) / 2 for (x, y) in zip(pred, y)]
            # print(math.exp(eta))
            dist = [p * math.exp(eta * r) for (p, r) in zip(dist, predVanilla)]
            dist = normalizeDist(dist)
            # score = self._tree.score(boostFeatures, y)
            score = accuracy_score(y, self.predict(X))
            # scoreTree = accuracy_score(y, self._tree.predict(boostFeatures))
            self._boostAcc = score
            # print(max(dist))
            # print(X)
            # print(y)
            # print(boostFeatures)
            printFunc("Current boosted accuracy: " + '{:.2f}'.format(score))
            # printFunc("Current tree accuracy: " + '{:.2f}'.format(scoreTree))
        printFunc("")
        printFunc("--> Final boosted accuracy: " + '{:.2f}'.format(score))
        return self

    def predict(self, X):
        """ Predict class labels for X.
        Parameters
        ----------
        X : {array-like, sparse matrix}, shape = [n_samples, n_features]
            Training vectors, where n_samples is the number of samples and
            n_features is the number of features.
        Returns
        ----------
        maj : array-like, shape = [n_samples]
            Predicted class labels.
        """

        check_is_fitted(self, '_estimators')

        features = []
        for est in self._estimators:
            features.append(est.predict(X))
        # print("encoded features list")
        # print(features)
        boostFeatures = np.asarray(features)
        boostFeatures = np.transpose(boostFeatures)
        # print("encoded features matrix")
        # print(boostFeatures)
        predTree = self._tree.predict(boostFeatures)

        return predTree

    def _collect_probas(self, X):
        """Collect results from clf.predict calls. """
        raise NotImplementedError
        return np.asarray([clf.predict_proba(X) for clf in self.estimators])

    def _predict_proba(self, X):
        """Predict class probabilities for X in 'soft' voting """
        raise NotImplementedError
        if self.voting == 'hard':
            raise AttributeError("predict_proba is not available when"
                                 " voting=%r" % self.voting)
        check_is_fitted(self, 'estimators')
        avg = np.average(self._collect_probas(X), axis=0, weights=self.weights)
        return avg

    @property
    def predict_proba(self):
        """Compute probabilities of possible outcomes for samples in X.
        Parameters
        ----------
        X : {array-like, sparse matrix}, shape = [n_samples, n_features]
            Training vectors, where n_samples is the number of samples and
            n_features is the number of features.
        Returns
        ----------
        avg : array-like, shape = [n_samples, n_classes]
            Weighted average probability for each class per sample.
        """
        raise NotImplementedError
        return self._predict_proba

    def transform(self, X):
        """Return class labels or probabilities for X for each estimator.
        Parameters
        ----------
        X : {array-like, sparse matrix}, shape = [n_samples, n_features]
            Training vectors, where n_samples is the number of samples and
            n_features is the number of features.
        Returns
        -------
        If `voting='soft'`:
          array-like = [n_classifiers, n_samples, n_classes]
            Class probabilities calculated by each classifier.
        If `voting='hard'`:
          array-like = [n_samples, n_classifiers]
            Class labels predicted by each classifier.
        """
        raise NotImplementedError
        check_is_fitted(self, 'estimators')
        if self.voting == 'soft':
            return self._collect_probas(X)
        else:
            return self._predict(X)

    def _predict(self, X):
        """Collect results from clf.predict calls. """
        raise NotImplementedError
        return np.asarray([clf.predict(X) for clf in self.estimators]).T


    def score(self, X, y, sample_weight=None):
        """Returns the mean accuracy on the given test data and labels.

        In multi-label classification, this is the subset accuracy
        which is a harsh metric since you require for each sample that
        each label set be correctly predicted.

        Parameters
        ----------
        X : array-like, shape = (n_samples, n_features)
            Test samples.

        y : array-like, shape = (n_samples) or (n_samples, n_outputs)
            True labels for X.

        sample_weight : array-like, shape = [n_samples], optional
            Sample weights.

        Returns
        -------
        score : float
            Mean accuracy of self.predict(X) wrt. y.

        """
        return accuracy_score(y, self.predict(X), sample_weight=sample_weight)