/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                           */
/*    This file is part of the program ipboost                               */
/*                                                                           */
/*    an implementation of a boosting learning algorithm.                    */
/*                                                                           */
/*    Copyright (C) 2016-                                                    */
/*                                                                           */
/*                                                                           */
/*    Based on SCIP  --- Solving Constraint Integer Programs                 */
/*       SCIP is distributed under the terms of the SCIP Academic Licence,   */
/*       see file COPYING in the SCIP distribution.                          */
/*                                                                           */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/**@file   pricer_learning.h
 * @brief  variable pricer for generating learning data with boosting
 */

/*---+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8----+----9----+----0----+----1----+----2*/

#ifndef __SCIP_PRICER_LEARNING__
#define __SCIP_PRICER_LEARNING__

#include "probdata_ipboost.h"

#ifdef __cplusplus
extern "C" {
#endif

/** creates the learning variable pricer and includes it in SCIP */
extern
SCIP_RETCODE SCIPincludePricerLearning(
   SCIP*                 scip                /**< SCIP data structure */
   );

/** adds variable based on passed information */
extern
SCIP_RETCODE SCIPaddLearnerVar(
   SCIP*                 scip,               /**< SCIP data structure */
   int                   dim,                /**< dimension */
   int                   npoints,            /**< number of points */
   double*               dist,               /**< weights for variable */
   double*               pred,               /**< predictions for variable */
   SCIP_Bool             initial             /**< whether the variable is initial */
   );

#ifdef __cplusplus
}
#endif

#endif
