/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                           */
/*    This file is part of the program ipboost                               */
/*                                                                           */
/*    an implementation of a boosting learning algorithm.                    */
/*                                                                           */
/*    Copyright (C) 2016-                                                    */
/*                                                                           */
/*                                                                           */
/*    Based on SCIP  --- Solving Constraint Integer Programs                 */
/*       SCIP is distributed under the terms of the SCIP Academic Licence,   */
/*       see file COPYING in the SCIP distribution.                          */
/*                                                                           */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/**@file   readargs.h
 * @brief  read comand line arguments
 */

#ifndef __READARGUMENTS_H__
#define __READARGUMENTS_H__

#include <scip/scip.h>


/** get problem name */
extern
SCIP_RETCODE getProblemName(
   const char*          filename,            /**< filename */
   char*                probname,            /**< returned problem name */
   int                  maxsize              /**< maximal size of probname */
   );


/** read comand line arguments */
extern
SCIP_RETCODE readArguments(
   int                   argc,               /**< number of shell parameters */
   const char**          argv,               /**< array with shell parameters */
   const char**          probname,           /**< problem name from arguments */
   const char**          testname,           /**< testing file name (optional) */
   const char**          settingsname,       /**< name of settings file */
   SCIP_Real*            timelimit,          /**< time limit read from arguments */
   SCIP_Real*            memlimit,           /**< memory limit read from arguments */
   SCIP_Longint*         nodelimit,          /**< node limit read from arguments */
   int*                  dispfreq,           /**< display frequency */
   int*                  seed                /**< random seed */
   );

#endif
